# Generated by Django 2.1.2 on 2019-01-08 14:33

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='FeatureSwitch',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('feature', models.CharField(max_length=100)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='activated_features', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'db_table': 'features_user',
            },
        ),
        migrations.AlterUniqueTogether(
            name='featureswitch',
            unique_together={('feature', 'user')},
        ),
    ]
